package com.jiuqi.sql;

import java.util.ArrayList;
import java.util.List;

public class SQLPretreatment {
   private String originSQL;
   private String sql;
   private List params;
   private boolean toUpperCase = false;
   private boolean ignoreComment = false;
   private static final char[] BLANK_SET = new char[]{' ', '\t', '\r', '\n'};

   public SQLPretreatment(String sql) {
      this.originSQL = sql;
      sql = null;
      this.params = new ArrayList();
   }

   public void execute() throws DBException {
      this.reset();
      StringBuffer sqlBuffer = new StringBuffer();
      this.build(sqlBuffer);
      this.sql = sqlBuffer.toString().trim();
   }

   public String getSQL() {
      return this.sql;
   }

   public String[] getParamNames() {
      return (String[])this.params.toArray(new String[0]);
   }

   public void setToUpperCase(boolean toUpperCase) {
      this.toUpperCase = toUpperCase;
   }

   public boolean getToUpperCase() {
      return this.toUpperCase;
   }

   public void setIgnoreComment(boolean ignoreComment) {
      this.ignoreComment = ignoreComment;
   }

   public boolean getIgnoreComment() {
      return this.ignoreComment;
   }

   private void reset() {
      this.sql = null;
      this.params.clear();
   }

   private char convertChar(char ch) {
      return this.toUpperCase ? Character.toUpperCase(ch) : ch;
   }

   private void build(StringBuffer buffer) throws DBException {
      CharReader reader = new CharReader((CharReader)null);

      while(!reader.isEnd()) {
         char ch = reader.read();
         switch (ch) {
            case '\u0000':
            case ';':
               return;
            case '\'':
               buffer.append(ch);
               if (!this.readToStringEnd(reader, buffer)) {
                  throw new DBException("SQL语法错误：含有未结束的字符串！");
               }
               break;
            case '-':
               if (reader.peek() == '-') {
                  reader.read();
                  if (!this.ignoreComment) {
                     buffer.append("--");
                  }

                  this.gotoNextLine(reader, buffer);
               } else {
                  buffer.append(ch);
               }
               break;
            case '/':
               if (reader.peek() == '*') {
                  reader.read();
                  if (!this.ignoreComment) {
                     buffer.append("/*");
                  }

                  this.gotoCommentEnd(reader, buffer);
               } else {
                  buffer.append(ch);
               }
               break;
            case '?':
               buffer.append(ch);
               String paramName = this.readParamName(reader);
               this.params.add(paramName);
               break;
            case 'G':
            case 'g':
               if (reader.check(-1, BLANK_SET) && reader.check(0, new char[]{'o', 'O'}) && reader.check(1, BLANK_SET)) {
                  return;
               }

               buffer.append(this.convertChar(ch));
               break;
            default:
               buffer.append(this.convertChar(ch));
         }
      }

   }

   private void gotoNextLine(CharReader reader, StringBuffer buffer) {
      while(true) {
         if (!reader.isEnd()) {
            char ch = reader.read();
            if (ch != '\r' && ch != '\n') {
               if (!this.ignoreComment) {
                  buffer.append(ch);
               }
               continue;
            }

            buffer.append(ch);
         }

         return;
      }
   }

   private void gotoCommentEnd(CharReader reader, StringBuffer buffer) {
      while(true) {
         if (!reader.isEnd()) {
            char ch = reader.read();
            if (ch != '*' || reader.peek() != '/') {
               if (!this.ignoreComment) {
                  buffer.append(ch);
               }
               continue;
            }

            reader.read();
            buffer.append(this.ignoreComment ? " " : "*/");
         }

         return;
      }
   }

   private boolean readToStringEnd(CharReader reader, StringBuffer buffer) {
      char ch;
      do {
         if (reader.isEnd()) {
            return false;
         }

         ch = reader.read();
         buffer.append(ch);
      } while(ch != '\'');

      return true;
   }

   private String readParamName(CharReader reader) {
      StringBuffer paramName = new StringBuffer();

      while(!reader.isEnd()) {
         char ch = reader.peek();
         if ((ch < 'a' || ch > 'z') && (ch < 'A' || ch > 'Z') && (ch < '0' || ch > '9') && ch != '_' && ch != '@') {
            break;
         }

         paramName.append(ch);
         reader.read();
      }

      return paramName.toString();
   }

   private class CharReader {
      private int curPos;
      public static final char EOF = '\u0000';

      private CharReader() {
         this.curPos = 0;
      }

      public boolean isEnd() {
         return SQLPretreatment.this.originSQL == null || this.curPos >= SQLPretreatment.this.originSQL.length();
      }

      public char read() {
         return this.isEnd() ? '\u0000' : SQLPretreatment.this.originSQL.charAt(this.curPos++);
      }

      public char peek() {
         return this.isEnd() ? '\u0000' : SQLPretreatment.this.originSQL.charAt(this.curPos);
      }

      public char peekNext() {
         if (SQLPretreatment.this.originSQL == null) {
            return '\u0000';
         } else {
            return this.curPos + 1 >= SQLPretreatment.this.originSQL.length() ? '\u0000' : SQLPretreatment.this.originSQL.charAt(this.curPos + 1);
         }
      }

      public char peek(int offset) {
         if (SQLPretreatment.this.originSQL == null) {
            return '\u0000';
         } else {
            int newPos = this.curPos + offset;
            return newPos >= 0 && newPos < SQLPretreatment.this.originSQL.length() ? SQLPretreatment.this.originSQL.charAt(newPos) : '\u0000';
         }
      }

      public boolean check(int offset, char[] charSet) {
         if (charSet == null) {
            return false;
         } else {
            char ch = this.peek(offset);

            for(int i = 0; i < charSet.length; ++i) {
               if (charSet[i] == ch) {
                  return true;
               }
            }

            return false;
         }
      }

      // $FF: synthetic method
      CharReader(CharReader var2) {
         this();
      }
   }
}
